
	List p=16F628A
 	Include p16F628A.inc
	__config 	_BODEN_ON & _CP_OFF & _PWRTE_ON & _MCLRE_ON & _INTRC_OSC_NOCLKOUT & _WDT_ON & _LVP_OFF	

	;*************************************************************
	;* Control software for M50555-001SP Character generator     *
	;*               Ver 1.1		06-09-02 
	;*	Updated for PIC16F628A 26 July 2004                  *
	;*************************************************************

	;*************************************************************
	;*                         Credits                           *
	;* AT keyboard routine originaly developed by Peter Luethi   *
	;*          http://www.electronic-engineering.ch             *
	;* Most routines have been heavily butchered in this         *
	;*                        application!                       *
 	;*************************************************************
 	
	;**********************
	;*** DATA LOCATIONS ***
	;**********************

	cblock	h'20'
	
	Loop_Count
	Serial_data          	
	Display_Count
	Display_Data_1
	Display_Data_2
	Display_Data_Count
	Pointer

	Page_Number
	Char_Poss

	I2C_Data
	I2C_Slave_Addr
	Eeprom_Addr
	
	Temp_1			;
	Temp_2			; 
	Temp_3			; 
	Temp_4			; 

	Flags_1			; register containing keyboard and other flags
	Flags_2			;

	KBDcnt			; IRQ based keyboard scan pattern counter
	KBD				; keyboard scan code & ascii data register

	W_Temp			; context register (ISR)
	STATUS_Temp 	; context register (ISR)
	PCLATH_Temp 	; context register (ISR)
	FSR_Temp 		; context register (ISR)

	endc

	;*******************
	;*** BIT EQUATES ***
	;*******************

	#define Sync_Pin	PORTA,0		; High when sync present
	#define	KBDdatapin 	PORTA,1		; keyboard data input port
	#define	Sda			PORTA,2
	#define	Scl			PORTA,3
	
	#define	KBDclkpin 	PORTB,0		; keyboard clock input port (IntB)
	#define	Clock_Pin 	PORTB,1
	#define	Data_Pin 	PORTB,2
	#define	Enable_Pin	PORTB,3
	#define	Test_Pin	PORTB,4
	#define	Switch_Pin	PORTB,5
	#define	Reset_Pin	PORTB,6
	
	#define	c			STATUS,0        ; carry bit in status
	#define	z			STATUS,2 		; zero bit in status
	#define	rp0 		STATUS,5        ; register select bit in status

	#define	RELflag		Flags_1,0	; release flag (0xF0)
	#define	SHIflag		Flags_1,1	; shift flag (0x12 / 0x59)
	#define	SPEflag		Flags_1,2	; special code flag (0xE0)
	#define	CAPflag		Flags_1,3	; caps lock flag (0x0D)
	#define	Switch 		Flags_1,4	; Switch pressed 
	#define	No_Blink 	Flags_1,5	; Stears flash curser routine
	#define	KBDflag		Flags_1,6	; keyboard data reception flag
	#define	Edit_Flag	Flags_1,7	; When set indicates edit mode

	#define	Full_Screen_Enable	Flags_2,0	; Set when no ext sync
	#define	Help_Page_Flag		Flags_2,1	; Set to indicate help page displayed

	#define	KBDdatatris 	TRISA,1
	#define	KBDclktris 	TRISB,0

	;********************
	;*** LABEL VALUES ***
	;********************

Version_No			equ	.11		; Used to check for valid Eeprom data
Vesion_No_Addr		equ	h'ff'		; Position in eeprom where vesion No is stored
Page_Number_Addr	equ	h'fe'		; Position in eeprom where Page Number is stored

Device_Base_Addr	equ	b'10100000'	; Eeprom base address

I2C_Port_Tris		equ	PORTA
Scl_Out_Sda_Out		equ	b'11110011'	
Scl_Out_Sda_In		equ	b'00000100'	

;----------------------------------------------------------------------;
BANK1	macro
	bsf	STATUS,RP0
	endm
	
BANK0	macro
	bcf	STATUS,RP0
	endm

;----------------------------------------------------------------------;
	ORG	0x00
	goto	Initialise

;----------------------------------------------------------------------;
	ORG	0x04		; interrupt vector location

ISR	;************************
	;*** ISR CONTEXT SAVE ***
	;************************
	bcf	INTCON,GIE	; disable all interrupts
	btfsc	INTCON,GIE	; assure interrupts are disabled
	goto	ISR
	movwf	W_Temp		; context save: W
	swapf	STATUS,W	; context save: STATUS
	movwf	STATUS_Temp	; context save
	clrf	STATUS		; bank 0, regardless of current bank
	movfw	PCLATH		; context save: PCLATH
	movwf	PCLATH_Temp	; context save
	clrf	PCLATH		; page zero, regardless of current page
	bcf	STATUS,IRP	; return to bank 0
	movfw	FSR		; context save: FSR
	movwf	FSR_Temp	; context save
	;*** context save done ***

	; Identify interupt source
	btfsc	INTCON,INTF		; Check Rb0 interupt
	goto	Check_Start_Bit
	btfss	INTCON,T0IE
	goto	Check_RBIF
	btfsc	INTCON,T0IF		; Check for timer interupt
	goto	Timer_Overflow		
Check_RBIF
	btfss	INTCON,RBIE
	goto	ISRend
	btfsc	INTCON,RBIF		; Check for Switch press (Rb4-8 change interupt)
	goto	Switch_Detected
	goto	ISRend			; Source not detected so abort

	;*****************************************
	;*** KEYBOARD SCAN PATTERN ACQUISITION ***
	;*****************************************

Check_Start_Bit
	;*** check start bit ***
	tstf	KBDcnt		; check
	bnz	_KBDdat		; branch on no zero
	btfsc	KBDdatapin	; test start bit of keyboard data input
	goto	_KBDabort	; no valid start bit, abort
	goto	_INCF		; exit

	;*** keyboard scan pattern acquisition ***
_KBDdat	movfw	KBDcnt		; get kbd scan pattern acquisition counter
	sublw	d'8'		; w = d'8' - KBDcnt (*)
	bnc	_KBDpari	; branch if negative (carry == 0)
	btfss	KBDdatapin	; get keyboard data input
	bcf	KBD,0x07	; (bit operations do not alter zero flag)
	btfsc	KBDdatapin
	bsf	KBD,0x07	
	bz	_INCF		; exit on zero (zero flag still valid from (*))
	rrf	KBD,F		; do this only 7 times
	goto	_INCF		; exit

	;*** ignore parity bit ***
_KBDpari movfw	KBDcnt		; get kbd scan pattern acquisition counter
	sublw	d'9'		; w = d'9' - KBDcnt
	bnc	_KBDstp		; branch if negative (carry == 0)
	goto	_INCF		; exit

	;*** check stop bit ***
_KBDstp	btfss	KBDdatapin	; check if stop bit is valid
	goto	_KBDabort	; if not set, abort
	bsf	KBDflag		; else set reception flag to decode KBD
	;*** stall keyboard ***
	; to prevent the arrival of more data before having finished decoding
	BANK1			; hold keyboard (with kbd clk low):
	bcf	KBDclktris	; set clk line to output
	BANK0
	bcf	KBDclkpin	; set keyboard clk line low (stall)
	;*** disable RB0/INT interrupt line ***
	bcf	INTCON,INTE	; disable RB0/INT interrupt
	goto	_KBDterm	; terminate successfully

_KBDabort clrf	KBD		; abort / invalid data
_KBDterm clrf	KBDcnt		; reset kbd scan pattern acquisition counter
	goto	_KBDend		; terminate execution of keyboard ISR
_INCF	incf	KBDcnt,F	; increment acquisition counter
_KBDend	bcf	INTCON,INTF	; clear RB0/INT interrupt flag
	goto	ISRend		; terminate execution of ISR

	;*****************************************
	;*** SWITCH DETECTED, SET UP TIMER     ***
	;*****************************************
Switch_Detected
	; Set up 20ms time delay
	bcf	INTCON,RBIE	; Stop further PORTB change interupts
	movlw	.178
	movwf	TMR0		; Setup timer 0 for 20ms approx	
	bcf	INTCON,T0IF	; Clear timer 0 interupt flag
	bsf	INTCON,T0IE	; Enable timer 0 interupts 
	goto	ISRend

	;*****************************************
	;*** TIMER OVERFLOW, SWITCH ROUTINE    ***
	;*****************************************
Timer_Overflow
	; 20ms time delay up
	bcf	INTCON,T0IE	; Disable timer 0 interupts 
	; Is switch still pressed?
	btfsc	Switch_Pin
	goto	Timer_Overflow_1
	bsf	Switch		; Indicate called by switch
	call	Inc_Page
	bcf	Switch
Timer_Overflow_1
	bcf	INTCON,T0IF
	bcf	INTCON,RBIF	; Clear PORTB change Interupt flag
	bsf	INTCON,RBIE	; Enable interupt on PORTB change
;	goto	ISRend

	;*****************************************
	;*** ISR TERMINATION (CONTEXT RESTORE) ***
	;*****************************************

ISRend	movfw	FSR_Temp	; context restore
	movwf	FSR		; context restore
	movfw	PCLATH_Temp	; context restore
	movwf	PCLATH		; context restore
	swapf	STATUS_Temp,W	; context restore
	movwf	STATUS		; context restore
	swapf	W_Temp,F	; context restore
	swapf	W_Temp,W	; context restore
	RETFIE			; enable global interrupt (INTCON,GIE)

;***** END OF INTERRUPT SERVICE ROUTINE *****


KBDdecode
	;*** check key release scan code (F0) ***
	movfw	KBD		; get scan code
	sublw	0xF0		; check if FO has been sent:
	bnz	_KBD_1		; branch if no 'release' scan code occured
	bsf	RELflag		; set key release flag if 'release' occured
	bcf	SPEflag		; clear special code flag always on release
	goto	_ClrStall	; abort with nothing to display
_KBD_1	btfss	RELflag		; check release flag, exit if cleared:
	goto	_KBD_2		; release flag has not been set, branch
	;*** release flag has been set / key release is in progress: ***
	bcf	RELflag		; clear key release flag
	;*** if release of SHIFT key, clear shift flag: ***
	movfw	KBD		; check left shift button (0x12):
	sublw	0x12		; subtract, check if zero
	bz	_clrSHI		; if zero, branch
	movfw	KBD		; check right shift button (0x59):
	sublw	0x59		; subtract, check if zero
	skpnz			; skip on non zero
_clrSHI	bcf	SHIflag		; clear shift flag
	;*** check for CAPS LOCK activity: ***
	movfw	KBD		; check caps lock key release:
	sublw	0x58		; subtract, check if zero
	bnz	_ClrStall	; if not zero, branch
	btfss	CAPflag		; check flag, clear flag if set:
	goto	_setCAP		; flag has not been set, branch
	bcf	CAPflag		; clear flag
	goto	_ClrStall	; abort with nothing to display
_setCAP	bsf	CAPflag		; set flag if cleared
	goto	_ClrStall	; abort with nothing to display

	;********************************
	;*** SCAN CODE RANGE CHECKING ***
	;********************************

_KBD_2	; Is help screen displayed?
	btfsc	Help_Page_Flag
	goto	Clear_Help_Page	; Yes so clear it!
	;*** check if special code (0xE0) has been submitted previously ***
	btfss	SPEflag
	goto	_KBD_3
	;*** decoding of scan code with preceeding special code (0xE0) ***
	movfw	KBD
	sublw	0x4A		; 0x4A - w
	bnz	_KBD4		; branch on non-zero
	movlw	0xac		; store '/' in KBD
	movwf	KBD
	goto	_OUTP
_KBD4	movfw	KBD
	sublw	0x7d		; 0x7d - w Page up
	bz	Inc_Page	; branch on non-zero
	movfw	KBD
	sublw	0x7a		; 0x7a - w Page down
	bz	Dec_Page	; branch on non-zero
	movfw	KBD
	sublw	0x6b		; 0x6b - w Left arrow
	bz	Left_Arrow	; branch on non-zero
	movfw	KBD
	sublw	0x74		; 0x74 - w Right arrow
	bz	Right_Arrow	; branch on non-zero
	movfw	KBD
	sublw	0x75		; 0x75 - w Up arrow
	bz	Up_Arrow	; branch on non-zero
	movfw	KBD
	sublw	0x72		; 0x72 - w Down arrow
	bz	Down_Arrow	; branch on non-zero
	movfw	KBD
	sublw	0x6c		; 0x6c - w Home (clear page)
	bz	Clear_Page	; branch on non-zero
_NOSUP	;*** check if scan code 0x5A or smaller has occurred ***
	movfw	KBD
	sublw	0x5A		; 0x5A - w
	bc	_KBD_3		; carry if result positive or zero, branch
	goto	_ClrStall
_KBD_3	;*** check if scan code 0x84  or higher has occurred ***
	movfw	KBD
	sublw	0x84		; 
	bc	KBD_dec		; lower than 0x84
	;*** no character to display: ***
	;*** check for special code (0xE0) ***
	movfw	KBD
	sublw	0xe0		; 
	skpnz			; skip if not zero
	bsf	SPEflag		; special code occurred, set flag
	goto 	_ClrStall	; abort with nothing to display
	; Change displayed page

	;*******************************************************
	;*** SCAN CODE DECODING & ASCII CHARACTER CONVERSION ***
	;*******************************************************

	;*** DECODE SCAN CODE ***
KBD_dec
	movlw	.1			; Index into table correctly
	addwf	KBD,f
_CHARlup
    	movlw	LOW M50555_Table 	; address of table
	addwf	KBD,f			; add offset
	movlw	HIGH M50555_Table 	; get correct page for PCLATH
	btfsc	c			; check if over page boundary
	addlw	.1
	movwf	PCLATH			; prepare right page bits for table read
	movfw	KBD
	call	M50555_Table
	movwf	KBD			; save result to KBD
	tstf	KBD			; test KBD to get the zero flag
	bz	_ClrStall		; abort if KBD is zero / invalid entry, nothing to display
	; Check Caps lock and Shift status
	btfsc	CAPflag		; check caps lock flag
	goto	Caps_Set	; flag has been set 
	btfss	SHIflag		; check shift flag
	goto	_KBD_dec_End	; no flag, exit
	; If caps set then convert A - Z to upper case or look up shifted
Caps_Set
	movlw	0x24		; less than "a"?
	subwf	KBD,w
	bnc	Not_A_Z		; Yes
	movlw	0x3e		; More than "z"?
	subwf	KBD,w		; = to or > than (
	bc	Not_A_Z
	btfsc	SHIflag
	goto	Shift_Set_Check_Caps
	goto	Translate_Key_Case	; Only Caps lock set
Shift_Set_Check_Caps
	btfsc	CAPflag
	goto	_KBD_dec_End	; No translation as caps lock and shift set
Translate_Key_Case
	movlw	.26
	subwf	KBD,f		; Yes
	goto	_KBD_dec_End
Not_A_Z
	; Is shift set? If so parse KBD for shifted characters
	btfss	SHIflag
	goto	_KBD_dec_End	; Shift not pressed so output normal character
	; If shift set then lookup "top row" characters if not A - Z
	call	Shift_Key_Parse
	movwf	KBD		; Save returned byte (in w) to KBD
	tstf	KBD
	bz	_ClrStall	; zero returned - key not found
_KBD_dec_End
	goto	_OUTP		; Output typed character 

	;**********************************************************************
	;*** LOOKUP SHIFTED KEYS (Not elegant code but shorter than table!) ***
	;**********************************************************************

Shift_Key_Parse
	movfw	KBD
	sublw	0x09	; (
	btfsc	z
	retlw	0x3e
	movfw	KBD
	sublw	0x40	; )
	btfsc	z
	retlw	0x3f
	movfw	KBD
	sublw	0xae	; +
	btfsc	z
	retlw	0xaa
	movfw	KBD
	sublw	0x08	; *
	btfsc	z
	retlw	0xad
	movfw	KBD
	sublw	0xf7	; :
	btfsc	z
	retlw	0xf7
	movfw	KBD
	sublw	0x01	; !
	btfsc	z
	retlw	0xfb
	movfw	KBD
	sublw	0x03	; Stop symbol
	btfsc	z
	retlw	0xf5
	movfw	KBD
	sublw	0x04	; Play symbol
	btfsc	z
	retlw	0xf2
	movfw	KBD
	sublw	0x05	; Record symbol
	btfsc	z
	retlw	0xf1
	movfw	KBD
	sublw	0x06	; Pause symbol
	btfsc	z
	retlw	0xf6
	movfw	KBD
	sublw	0xed	; Clock symbol
	btfsc	z
	retlw	0xaf
	movfw	KBD
	sublw	0xab	; Hand pointing right
	btfsc	z
	retlw	0xfc
	movfw	KBD
	sublw	0xff	; Space
	btfsc	z
	retlw	0xfd	; Return special space
	movfw	KBD
	sublw	0x76	; Escape
	btfsc	z
	retlw	0x41
	retlw 	0	; Key not found

	;***********************************************
	;*** Clear help page and re-display continue ***
	;***********************************************

Clear_Help_Page
	bcf	Help_Page_Flag
	call	Load_OSD		; Re-display page	
	goto	_ClrStall


	;**********************************************************
	;*** STALL RELEASE & CLEAR KEYBOARD DATA RECEPTION FLAG ***
	;**********************************************************

_ClrStall 
	BANK1
	bsf	KBDclktris	; set clk line back to input (and goes high)
	BANK0			;	(release stall)
	bcf	KBDflag		; clear keyboard data reception flag
	bsf	INTCON,INTE	; re-enable interrupt RB0/INT
	RETURN

	;*************************************************
	;*** SPECIAL COMMANDS (without special output) ***
	;*************************************************

_SHIFT	bsf	SHIflag		; set shift flag
	RETLW	0		; clear w to obtain invalid entry

	;**********************************************
	;*** INITIALISATION - (Relocated in memory) ***
	;**********************************************

Initialise
	BANK1
	movlw  	b'11111111'
	movwf  	TRISA                	;tristate porta
	movlw  	b'11100001'
	movwf  	TRISB                	;tristate key port
	movlw	b'00000111'
	movwf	OPTION_REG 		; 7 - Port B pullups enabled
					; 6 - keyboard interrupt on falling edge (INTB)
					; 5 - TMR0 source internal clock
					; 4 - TMR0 signal edge select
					; 3 - Prescaler assigned to timer
					; 2-0 Prescaler 256
	BANK0
	movlw	h'07'
	movwf	CMCON			; Use port A as I/O
	clrf	KBDcnt			; clear IRQ based scan pattern counter
	clrf	Flags_1			; clear all flags (keyboard & other)
	clrf	Flags_2			; clear all flags 
	clrf	Page_Number
	clrf	Char_Poss		; Initialise character possition counter
	movlw	b'11111000'
	andwf	INTCON,F		; clear all interrupt flags
	bsf	Clock_Pin
	bsf	Enable_Pin
	; Set Device_Addr
	movlw	Device_Base_Addr
	movwf	I2C_Slave_Addr

	; Check for manual Eeprom initialisation request
	btfss	Reset_Pin		; Pin pulled low when reset requested
	call	Initialise_All

	;*****************************
	;*** EEPROM VALIDITY CHECK ***
	;*****************************

	; Check Eeprom for valid data
	movlw	Vesion_No_Addr
	movwf	Eeprom_Addr		; Load Eeprom address with version number location
	call	Random_Read
	movlw	Version_No
	subwf	I2C_Data,w
	btfsc	z			; Eeprom data valid, load saved data into OSD chip
	goto	No_Initialise
	;
	; Initialise all pages 
	call	Initialise_All

	; Initialise Version number
	movlw	Vesion_No_Addr
	movwf	Eeprom_Addr		; Load Eeprom address with version number location
	movlw	Version_No		; Load Eeprom data (Version No)
	movwf	I2C_Data
	call	Write_Byte		; Save to Eeprom
	;
No_Initialise
	; Load page number from Eeprom
	movlw	Page_Number_Addr	; Address in eeprom of page number
	movwf	Eeprom_Addr		; Load Eeprom address with page number location
	call	Random_Read
	movfw	I2C_Data
	movwf	Page_Number
	call	Page_Number_Check	
	;
Start
	call	Load_OSD		; Display last saved page 0
	;
	movlw	b'10011000'
	iorwf	INTCON,F		; enable global & RB0/INT interrupts
					; and PORTB change
_MLOOP
	clrwdt				; Clear watchdog timer
	btfsc	KBDflag			; check scan pattern reception flag
	call	KBDdecode		; if set, call decoding routine
	;
	btfsc	Sync_Pin		; Check for presence of sync
	goto	Sync_Present
	btfsc	Full_Screen_Enable	; Full screen should be set in abscence of sync
	goto	Sync_Status_OK
	bsf	Full_Screen_Enable
	call	Load_OSD		; Call OSD routine (sets full screen mode)
	goto	Sync_Status_OK
Sync_Present
	btfss	Full_Screen_Enable	; Is full screen flag set?
	goto	Sync_Status_OK
	btfsc	Help_Page_Flag
	goto	Sync_Status_OK
	bcf	Full_Screen_Enable
	call	Load_OSD		; Call OSD routine (recalls EX mode from memory)
Sync_Status_OK	
	goto	_MLOOP	

	;*************************
	;*** INITIALISE EEPROM ***
	;*************************

Initialise_All
	movlw	.8
	movwf	Temp_1			; Load temp_1 with page count
Init_1
	call	Initialise_Page
	movlw	.2
	addwf	I2C_Slave_Addr,f	; Add 2 as bit 0 is R/W bit
	decfsz	Temp_1,f		; Loop for other pages
	goto	Init_1
	movlw	Device_Base_Addr	; reset slave address to page 1
	movwf	I2C_Slave_Addr
	return


Initialise_Page
	; Initialise character area
	movlw	.240
	movwf	Temp_4
	clrf	Eeprom_Addr		; Start at 0
Initialise_Char_Loop			; Load default registers	
	movlw	0xff			; Load character area with spaces
	movwf	I2C_Data
	call	Write_Byte		; Write byte to Eeprom
	incf	Eeprom_Addr,f		; Increment address in Eeprom
	decfsz	Temp_4,f		; Next byte
	goto	Initialise_Char_Loop
	; Initialise registers
	movlw	.8
	movwf	Temp_4			; Loop counter
	clrf	Temp_3			; Index into table
Initialise_Reg_Loop
	movlw	HIGH OSD_Init_Table
	movwf	PCLATH
	movfw	Temp_3
	call	OSD_Init_Table
	movwf	I2C_Data
	call	Write_Byte		; Write byte to Eeprom
	incf	Eeprom_Addr,f		; Increment address in Eeprom
	incf	Temp_3,f
	decfsz	Temp_4,f		; Next byte
	goto	Initialise_Reg_Loop
	return

;	ORG 0x???			; Uncomment on assembly error!
OSD_Init_Table
	addwf	PCL,f		; Add offset to PCL
	retlw	b'00011101'	; H_Pos_LO		
	retlw	b'00000000'	; H_Pos_HI
	retlw	b'00011000'	; V_Pos_LO
	retlw	b'00000100'	; V_Pos_HI
	retlw	b'00010111'	; Video_Reg_LO
	retlw	b'00000000'	; Video_Reg_HI
	retlw	b'11011010'	; Cont_Reg_LO
OSD_Init_Table_End
	retlw	b'00001000'	; Cont_Reg_HI

	IF (high (OSD_Init_Table) != high (OSD_Init_Table_End))
		ERROR "Keyboard lookup table hits page boundary!"
	ENDIF


	;**************************
	;*** PAGE EDIT ROUTINES ***
	;**************************

Load_OSD
	; Loads current page for display or edit
	bcf	Enable_Pin		;Clear enable pin
	;
	movlw	.0			;Send start address
	movwf	Display_Data_1
	movlw	.0
	movwf	Display_Data_2
	call	Display_Out		
	;
	movlw	.0
	movwf	Eeprom_Addr
	movlw	.240
	movwf	Temp_4			; Load loop counter, 240 characters
Load_OSD_Char_Loop
	call	Random_Read
	movfw	I2C_Data
	btfss	Edit_Flag
	goto	No_Edit
	sublw	0xff			; Test for "space", (space converted to 40 hex)
	btfss	z
	goto	Not_A_Space
	movlw	0x6e			; Display  for spaces in edit
	goto	No_Edit
Not_A_Space
	movfw	I2C_Data
No_Edit
	movwf	Display_Data_1
	clrf	Display_Data_2
	call	Display_Out
	incf	Eeprom_Addr,f
	decfsz	Temp_4,f
	goto	Load_OSD_Char_Loop
	;
	movlw	.4
	movwf	Temp_4			; Load loop counter, 4 sets of registers
Load_OSD_Reg_Loop
	call	Random_Read
	movfw	I2C_Data
	movwf	Display_Data_1
	incf	Eeprom_Addr,f
	call	Random_Read
	movfw	I2C_Data
	movwf	Display_Data_2
	call	Display_Out
	incf	Eeprom_Addr,f
	decfsz	Temp_4,f
	goto	Load_OSD_Reg_Loop
	bsf	Enable_Pin		; Set enable pin
	;
	btfsc	Edit_Flag
	call	Flash_Curser
	btfss	Full_Screen_Enable	
	goto	Load_OSD_End
	; Retrieve existing data from register 246 in Eeprom (same as OSD)
	movlw	.246			; Get register from Eeprom
	call	Retrieve_Register
	bsf	I2C_Data,2		; Set EX bit
	movfw	I2C_Data
	movwf	Temp_3			; Save Data ready to be sent to OSD chip
	movlw	.247			; Get high byte from Eeprom
	call	Retrieve_Register
	movfw	I2C_Data
	movwf	Temp_4			; Save Data ready to be sent to OSD chip
	; Save data back to OSD chip
	bcf	Enable_Pin
	movlw	0xf3			; Location of register in OSD chip
	movwf	Display_Data_1
	clrf	Display_Data_2
	call	Display_Out		; Send address to OSD
	movfw	Temp_3
	movwf	Display_Data_1
	movfw	Temp_4
	movwf	Display_Data_2
	call	Display_Out		; Save data back
	bsf	Enable_Pin
Load_OSD_End
	return
	
Set_EX_Bit
	return
	
	;************************
	;*** DISPLAY ROUTINES ***
	;************************

Display_Out
	;	Clock out LO byte
	movlw	.8			;Loop 8 times
	movwf	Display_Count		;Load register
Display_Loop_1
	bcf	Clock_Pin		;Clear clock pin
	rrf	Display_Data_1,f	;Shift data 1 bit right
	bcf	Data_Pin		;Clear data pin
	btfsc	c			;Set data pin if carry set
	bsf	Data_Pin
	bsf	Clock_Pin		;Set clock pin - clock data out
	decfsz	Display_Count,f		
	goto	Display_Loop_1		;Loop until all clocked out
	;	Clock out HI byte
	movlw	.8			;Loop 8 times
	movwf	Display_Count		;Load register
Display_Loop_2
	bcf	Clock_Pin		;Clear clock pin
	rrf	Display_Data_2,f	;Shift data 2 byte right
	bcf	Data_Pin		;Clear data pin
	btfsc	c			;Set data pin if carry set
	bsf	Data_Pin
	bsf	Clock_Pin		;Set clock pin - clock data out
	decfsz	Display_Count,f
	goto	Display_Loop_2		;Loop until all clocked out
	return

	;****************************
	;*** KEY SERVICE ROUTINES ***
	;****************************

Inc_Page
	movlw	.2
	addwf	Page_Number,f
	goto	Check_Page
Dec_Page
	movlw	.2
	subwf	Page_Number,f
Check_Page
	call	Page_Number_Check
	movfw	I2C_Slave_Addr
	movwf	Temp_1
	; Save to eeprom
	movwf	I2C_Data
	movlw	Device_Base_Addr	; Change to page 0
	movwf	I2C_Slave_Addr
	movlw	Page_Number_Addr
	movwf	Eeprom_Addr		; Load Eeprom address with version number location
	call	Write_Byte		; Save to Eeprom
	; Restore page number
	movfw	Temp_1
	movwf	I2C_Slave_Addr
	clrf	Char_Poss		; Changed page so cursor position set to start
	call	Load_OSD
	btfss	Switch			; Called by interupt routine?
	goto 	_ClrStall
	return		

Page_Number_Check
	movfw	Page_Number
	andlw	b'00001110'		; Remove bit 0 and bits 4-7
	movwf	Page_Number		; Save back
	movlw	b'11110000'
	andwf	I2C_Slave_Addr,f	; Remove lower bits of Device address
	movfw	Page_Number
	iorwf	I2C_Slave_Addr,f	; "OR" page number and device address
	Return

Left_Arrow
	btfss	Edit_Flag		; Only works during edit mode
	goto	Left_Arrow_End
	movf	Char_Poss,f		; Check for start of screen
	bz	Left_Arrow_End
	bsf	No_Blink		; Clears flashing at current curser possition
	call	Flash_Curser
	decf	Char_Poss,f		; Moves left one space	
	call	Flash_Curser		; Recalls data at character possition and flashes
Left_Arrow_End
	goto 	_ClrStall

Right_Arrow
	btfss	Edit_Flag		; Only works during edit mode
	goto	Right_Arrow_End
	movlw	.239
	subwf	Char_Poss,w		; Check for end of screen
	bc	Right_Arrow_End	
	bsf	No_Blink
	call	Flash_Curser		; Clears flashing at current curser possition	
	incf	Char_Poss,f		; Moves left one space	
	call	Flash_Curser		; Recalls data at character possition and flashes
Right_Arrow_End
	goto 	_ClrStall

Up_Arrow	
	btfss	Edit_Flag		; Only works during edit mode
	goto	_ClrStall
	bsf	No_Blink
	call	Flash_Curser		; Clears flashing at current curser possition	
	movlw	.24
	subwf	Char_Poss,w		; Check for end of screen	
	bc	Up_Arrow_1
	movlw	.0			; As next line will put curser passed first screen 
	movwf	Char_Poss		; possition, set to last possition
	goto	Up_Arrow_2
Up_Arrow_1
	movlw	.24
	subwf	Char_Poss,f		; Move up one line by subtracting 24 character possitions	
Up_Arrow_2
	call	Flash_Curser		; Recalls data at character possition and flashes
	goto 	_ClrStall

Down_Arrow	
	btfss	Edit_Flag		; Only works during edit mod_Arrow_End
	goto	_ClrStall
	bsf	No_Blink
	call	Flash_Curser		; Clears flashing at current curser possition	
	movlw	.216
	subwf	Char_Poss,w		; Check for end of screen
	bnc	Down_Arrow_1
	movlw	.239			; As next line will put curser passed last screen 
	movwf	Char_Poss		; possition, set to last possition
	goto	Down_Arrow_2
Down_Arrow_1
	movlw	.24
	addwf	Char_Poss,f		; Move down one line by adding 24 character possitions	
Down_Arrow_2
	call	Flash_Curser		; Recalls data at character possition and flashes
	goto 	_ClrStall

_BCKSPC
	btfss	Edit_Flag		; Only works during edit mode
	goto	_BCKSPC_End
	movf	Char_Poss,f		; Check for start of screen
	bz	_BCKSPC_End
	decf	Char_Poss,f
	movlw	0xff
	movwf	KBD
	call	_OUTP
	bsf	No_Blink		; Indicate to overwrite blinking character
	call	Flash_Curser		; Overwrite
	decf	Char_Poss,f
	call	Flash_Curser		; Overwrite new character possition
_BCKSPC_End
	retlw	0	
	
_F1	; Help page
	bsf	Help_Page_Flag		; Indicate displaying help page
	bsf	Full_Screen_Enable	; Force full screen mode
	bcf	Enable_Pin		; Clear enable pin
	;
	movlw	.0			; Send start address
	movwf	Display_Data_1
	movlw	.0
	movwf	Display_Data_2
	call	Display_Out		
	;
	movlw	.240
	movwf	Temp_4			; Loop counter
	movlw	.1
	movwf	Temp_3			; Current possition in help page
Load_Help_Page_Loop
	clrwdt
	movfw	Temp_3
	movwf   Temp_1           	; Temp_1 holds possition in help page 
	movlw	LOW Help_Page		; Help page start address
	addwf	Temp_1,f		; Add offset
	movlw	HIGH Help_Page		; Load high byte of program counter
	btfsc	STATUS,C		; check if over page boundary
	addlw	1			; yes so inc high byte
	movwf	PCLATH
	movfw	Temp_1
	call	Help_Page
	movwf	Display_Data_1
	; ASCII caharacters need converting to M50555 character set
	; Check for "space"
	movlw	" "			;  "space"
	subwf	Display_Data_1,w
	movlw	h'ff'
	btfsc	z
	goto	Display_Help
	movlw	"="			;  "="
	subwf	Display_Data_1,w
	movlw	h'ae'
	btfsc	z
	goto	Display_Help
	movlw	"."			;  "."
	subwf	Display_Data_1,w
	movlw	h'ef'
	btfsc	z
	goto	Display_Help
	movlw	"+"			;  "+"
	subwf	Display_Data_1,w
	movlw	h'aa'
	btfsc	z
	goto	Display_Help
	movlw	"-"			;  "-"
	subwf	Display_Data_1,w
	movlw	h'ab'
	btfsc	z
	goto	Display_Help
	movlw	"0"			; Check for < character 0
	subwf	Display_Data_1,w
	bnc	Help_Page_Error		; Unsupported character
	movlw	":"
	subwf	Display_Data_1,w	; Check for number
	bc	Not_Number
	movlw	h'30'
	subwf	Display_Data_1,w	; Subtract Number offset
	goto	Display_Help
Not_Number
	movlw	h'40'
	subwf	Display_Data_1,w	; Check for punctuation
	bnc	Help_Page_Error		; Unsupported character
	movlw	h'5b'
	subwf	Display_Data_1,w	; Check for capital letters
	bc	Not_Uppercase
	movlw	.55			; Upper case offset
	subwf	Display_Data_1,w	; Subtract offset
	goto	Display_Help
Not_Uppercase
	movlw	h'60'
	subwf	Display_Data_1,w	; Check for punctuation
	bnc	Help_Page_Error		; Unsupported character
	movlw	h'7b'
	subwf	Display_Data_1,w	; Check for capital letters
	bc	Not_Uppercase
	movlw	("a"-.36)
	subwf	Display_Data_1,w	; Subtract offset
	goto	Display_Help	
Help_Page_Error	
	movlw	h'af'			; Indicate character not found
Display_Help
	movwf	Display_Data_1
	clrf	Display_Data_2
	call	Display_Out
	incf	Temp_3,f
	decfsz	Temp_4,f		; Decrement possition counter
	goto	Load_Help_Page_Loop
	; Load default registers
	movlw	.4
	movwf	Temp_4			; Loop counter, loop 4 times
	clrf	Temp_3			; Index into table
	movlw	HIGH Help_Reg_Table	; Load high byte of table
	movwf	PCLATH
Help_Page_Reg_Loop
	movfw	Temp_3
	call	Help_Reg_Table
	movwf	Display_Data_1
	incf	Temp_3,f
	movfw	Temp_3
	call	Help_Reg_Table
	movwf	Display_Data_2
	call	Display_Out
	incf	Temp_3,f
	decfsz	Temp_4,f		; Next 16 bit byte
	goto	Help_Page_Reg_Loop
	bsf	Enable_Pin		; Set enable pin
F1_End
	retlw	0

_F2	; Change background colour in full screen
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F2_End
	; Retrieve existing settings from dec 244 in Eeprom
	movlw	.244
	call	Retrieve_Register
	andlw	b'11111000'
	movwf	Temp_2			; Save other register bits
	incf	I2C_Data,w		; Increment chroma phase information
	andlw	b'00000111'		; Remove invalid bits
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F2_End
	retlw	0

_F3	; Change between full screen and auotomatic superimpose modes
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F3_End
	; Retrieve existing settings from dec 246 in Eeprom
	movlw	.246
	call	Retrieve_Register
	btfsc	I2C_Data,2		; Test EX bit in register
	goto	Clear_EX
	bsf	I2C_Data,2		; Set if cleared
	goto	EX_Done
Clear_EX
	bcf	I2C_Data,2		; Clear if set
EX_Done
	movfw	I2C_Data		; Required for "Save_Register" routine
	call	Save_Register
_F3_End
	retlw	0
	
_F4	
	retlw	0

_F5	; Change horizontal text possition
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F5_End
	; Retrieve existing settings from dec 242 in Eeprom
	movlw	.240
	call	Retrieve_Register
	andlw	b'11000000'
	movwf	Temp_2			; Save other register bits
	movlw	.2
	addwf	I2C_Data,w		; Add 2 to vertical register
	andlw	b'00111111'		; Remove invalid bits
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F5_End		
	retlw	0

_F6	; Change vertical text possition
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F6_End
	; Retrieve existing settings from dec 242 in Eeprom
	movlw	.242
	call	Retrieve_Register
	andlw	b'11100000'
	movwf	Temp_2			; Save other register bits
	movlw	.2
	addwf	I2C_Data,w		; Add 2 to vertical register
	andlw	b'00011111'		; Remove invalid bits
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F6_End	
	retlw	0

_F7	; Change character blanking
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F7_End
	; Retrieve existing settings from dec 246 in Eeprom
	movlw	.246
	call	Retrieve_Register
	andlw	b'10011111'
	movwf	Temp_2			; Save other register bits
	movlw	.32
	addwf	I2C_Data,w		; Add 32 to register
	andlw	b'01100000'		; Remove invalid bits
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F7_End
	retlw	0

_F8	; Change character mode
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F8_End
	; Retrieve existing settings from dec 245 in Eeprom
	movlw	.245
	call	Retrieve_Register
	andlw	b'11111000'
	movwf	Temp_2			; Save other register bits
	movlw	.1
	addwf	I2C_Data,w		; Add 1 to register
	andlw	b'00000111'		; Remove invalid bits
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F8_End
	retlw	0

_F9	; Change horizontal size of line 1
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F9_End
	; Retrieve existing settings from dec 240 in Eeprom
	movlw	.240
	call	Retrieve_Register
	andlw	b'00111111'
	movwf	Temp_2			; Save other register bits in Temp_2
	movlw	.64
	addwf	I2C_Data,w		; Increment horizontal size
	andlw	b'11000000'		; Remove invalid bits (shouldn't be any!)
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F9_End
	retlw	0

_F10	; Change Vertical size of line 1
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F10_End
	; Retrieve existing settings from dec 242 in Eeprom
	movlw	.242
	call	Retrieve_Register
	andlw	b'00111111'
	movwf	Temp_2			; Save other register bits in Temp_2
	movlw	.64
	addwf	I2C_Data,w		; Increment vertical size
	andlw	b'11000000'		; Remove invalid bits (shouldn't be any!)
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F10_End
	retlw	0

_F11	; Change horizontal size of line 2 to 10
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F11_End
	; Retrieve existing settings from dec 241 in Eeprom
	movlw	.241
	call	Retrieve_Register
	andlw	b'11111100'
	movwf	Temp_2			; Save other register bits in Temp_2
	movlw	.1
	addwf	I2C_Data,w		; Increment horizontal size
	andlw	b'00000011'		; Remove invalid bits 
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F11_End
	retlw	0

_F12	; Change Vertical size of lines 2 to 10 (oviously can't display all expanded!)
	btfss	Edit_Flag		; Only works during edit mode
	goto	_F12_End
	; Retrieve existing settings from dec 243 in Eeprom
	movlw	.243
	call	Retrieve_Register
	andlw	b'11111100'
	movwf	Temp_2			; Save other register bits in Temp_2
	movlw	.1
	addwf	I2C_Data,w		; Increment vertical size
	andlw	b'00000011'		; Remove invalid bits 
	iorwf	Temp_2,w		; "OR" with other register bits
	call	Save_Register
_F12_End
	retlw	0

Retrieve_Register
	movwf	Eeprom_Addr
	call	Random_Read
	movfw	I2C_Data
	return

Save_Register
	; Re-save back to possition in Eeprom and re-display
	movwf	I2C_Data			
	call	Write_Byte	
	call	Load_OSD		; Re-display page	
	return

_ESCAPE
	; Changes between edit and display modes
	btfsc	Edit_Flag
	goto	Remove_Edit
	bsf	Edit_Flag
	call	Load_OSD
	goto	_Escape_End
Remove_Edit
	bcf	Edit_Flag
	call	Load_OSD
_Escape_End
	retlw	0

Clear_Page
	; Clears page to defaults
	btfss	Edit_Flag		; Test for edit mode, only clears while editing
	goto	Clear_Page_End
	call	Initialise_Page
	call	Load_OSD		; Re display Eeprom contents, IE nothing!
Clear_Page_End
	goto	_ClrStall
	
_OUTP	; Save typed character to Eeprom if in edit mode, otherwise dump them!
	btfss	Edit_Flag
	goto	_ClrStall
	; Maximum character possition (0 - 239) reached yet?
	movlw	.240
	subwf	Char_Poss,w
	btfsc	c
	goto	_ClrStall		; Abort if character 240 or above
	; Store typed character
	movfw	Char_Poss		; Character possition same as Eeprom address
	movwf	Eeprom_Addr
	movlw	0x40			; Check for 40 hex, if present
	subwf	KBD,w			; clear KBD to display 0
	skpnz
	clrf	KBD
	movfw	KBD
	movwf	I2C_Data
	call	Write_Byte		; Write byte to Eeprom
	; Display typed character
	; Output address to OSD chip
	bcf	Enable_Pin
	movfw	Char_Poss
	movwf	Display_Data_1
	clrf	Display_Data_2
	call	Display_Out
	; Output typed character
	movlw	0x40			; Check for 40 hex, if present
	subwf	KBD,w			; clear KBD to display 0
	skpnz
	clrf	KBD
	movfw	KBD
	sublw	0xff			; Test for "space", (space converted to 40 hex)
	btfss	z
	goto	_OUTP2
	movlw	0x6e			; Display  for spaces in edit
	goto	_OUTP1
_OUTP2
	movfw	KBD
_OUTP1
	movwf	Display_Data_1
	movlw	b'00000000'
	movwf	Display_Data_2
	call	Display_Out
	incf	Char_Poss,f
	bsf	Enable_Pin
	call	Flash_Curser		; Flash curser position in edit mode
	goto 	_ClrStall

Flash_Curser
	; Retrieve character at curser from Eeprom
	movfw	Char_Poss
	movwf	Eeprom_Addr
	call	Random_Read
	; Set possition of character in OSD chip
	bcf	Enable_Pin
	movfw	Char_Poss
	movwf	Display_Data_1
	clrf	Display_Data_2
	call	Display_Out
	; Display character flashing or not depending on No_Blink flag
	movfw	I2C_Data
	sublw	0xff			
	btfss	z			; Test for "space", (space converted to 40 hex)
	goto	Display_Curser
	movlw	0x6e			; Display  for spaces in edit
	goto	Display_Curser_1
Display_Curser
	movfw	I2C_Data
Display_Curser_1
	movwf	Display_Data_1
	movlw	b'00001000'		; Set to flashing character 
	btfsc	No_Blink
	movlw	b'00000000'		; Set to non flashing character IE overwrite
	movwf	Display_Data_2
	call	Display_Out
	bsf	Enable_Pin
	bcf	No_Blink
	return

;******************************************************************************
;		I2C Routines
;******************************************************************************

I2C_Start     
	; I2C start, sent at start of new comunication
 	bsf    	Sda     		;set output latch
	movlw  	Scl_Out_Sda_Out   	;set scl and sda as output
	bsf    	rp0
	andwf  	I2C_Port_Tris,f           
	bcf    	rp0
	bsf    	Sda     
	call	I2C_Delay
	bsf    	Scl
	call	I2C_Delay
	bcf    	Sda       	;set start condition
	call	I2C_Delay
	bcf    	Scl
	return

I2C_Tx        
	; I2C transmit, data to be transmitted is in I2C_Data
 	movlw  	Scl_Out_Sda_Out   	;set scl and sda
	BANK1
	andwf  	I2C_Port_Tris,f        	;as output
	BANK0
	movlw  	.8              	;loop 8 times (8 bits)
	movwf  	Loop_Count
I2C_Tx_Loop    
	rlf    	I2C_Data,f     		;rotate into c
	btfsc  	c
	goto	Set_Sda
	bcf	Sda
	goto	Continue
Set_Sda
	bsf    	Sda
Continue
	call	I2C_Delay
	bsf    	Scl       	;clock data bit 
	call	I2C_Delay
	bcf    	Scl      	;out
	decfsz 	Loop_Count,1
	goto   	I2C_Tx_Loop
	movlw  	Scl_Out_Sda_In   	;set sda as
	bsf    	rp0
	iorwf  	I2C_Port_Tris,f        ;input to allow for
	bcf    	rp0
	bsf    	Scl     	;aknowledge
	bsf	c
	call	I2C_Delay
	btfss	Sda
	bcf	c		;clear c if aknowledge
	bcf    	Scl
       	return
I2C_Rc
	; I2C receive, data returned in I2C_Data
 	movlw  	Scl_Out_Sda_In   	;set scl out and sda in
	bsf    	rp0
	iorwf  	I2C_Port_Tris,f           
	bcf    	rp0
	movlw  	.8              	;loop 8 times (8 bits)
	movwf  	Loop_Count
I2C_Rc_loop    
	bsf    	Scl
	bsf  	c
	call	I2C_Delay
	btfss  	Sda
	bcf    	c
	call	I2C_Delay
	bcf    	Scl      	
	rlf    	I2C_Data,f      	;rotate c into I2C_Data
	decfsz 	Loop_Count,1
	goto   	I2C_Rc_loop
	return

I2C_No_Aknow
	; no aknowledge bit, used at the end of a receive before stop bit
 	movlw  	Scl_Out_Sda_In   	;set sda as
	bsf    	rp0		;input to allow for
	iorwf  	I2C_Port_Tris,f       	;aknowledge
	bcf    	rp0
	bsf    	Scl     	;send clock (scl)
	call	I2C_Delay
	bcf    	Scl		;high impedance
	return

I2C_Aknow
	; aknowledge bit, used to aknowlede byte of data correctly received
 	movlw  	Scl_Out_Sda_Out  	; set sda as output
	bsf    	rp0			; so aknowledge can be
 	andwf  	I2C_Port_Tris,f  	; sent
 	bcf    	rp0
	bcf	Sda			; aknowledge bit
	call	I2C_Delay
	bsf    	Scl     		; clock
	call	I2C_Delay
	bcf    	Scl			; out
	return

I2C_Stop      
	; stop bit, used to halt I2C comunication and frees bus
 	movlw  	Scl_Out_Sda_Out   	;set scl and
	bsf    	rp0
	andwf  	I2C_Port_Tris,f		;sda as output
	bcf    	rp0
	bcf    	Sda
	call	I2C_Delay
	bsf    	Scl
	call	I2C_Delay
	bsf    	Sda
	movlw  	Scl_Out_Sda_In     	;send sda to high imp.
	bsf    	rp0
	iorwf  	I2C_Port_Tris,f           
	bcf    	rp0
	return
I2C_Delay
	nop
	return
		
;******************************************************************************
Random_Read	
	clrwdt				; Clear watchdog... may have to wait!
	call	I2C_Start
	movfw	I2C_Slave_Addr
	movwf	I2C_Data
	bcf	I2C_Data,0		;set to write
	call	I2C_Tx
	btfsc	STATUS,C		;check carry flag
	goto	retry_Read_Eeprom	;no aknowledge so retry
	movfw	Eeprom_Addr
	movwf	I2C_Data		;start at address 00 in eeprom
	call	I2C_Tx
	btfsc	STATUS,C		;check c flag
	goto	retry_Read_Eeprom	;no aknowledge so retry
	call	I2C_Start	
	movfw	I2C_Slave_Addr
	movwf	I2C_Data
	bsf	I2C_Data,0		;set to read
	call	I2C_Tx
	btfsc	STATUS,C		;check carry flag
	goto	retry_Read_Eeprom	;no aknowledge so retry		
read_loop
	call	I2C_Rc
 	call	I2C_No_Aknow
	call	I2C_Stop
	return
retry_Read_Eeprom
	call	I2C_Stop
	goto	Random_Read

Write_Byte
	; Data transferred in I2C_Data, device address in I2C_Slave_Addr, 
	; possition in Eeprom in Eeprom_Addr
	clrwdt				; Clear watchdog... may have to wait!
	movfw	I2C_Data
	movwf	Temp_2
Write_Retry		
	call	I2C_Start
	movfw	I2C_Slave_Addr
	movwf	I2C_Data
	bcf	I2C_Data,0		;sets to write
	call	I2C_Tx
	btfsc	c			;check to see if aknowledge
	goto	I2C_Busy		;bit set (0). if not busy
	movfw	Eeprom_Addr		;address for data in device
	movwf	I2C_Data
	call	I2C_Tx			;load address in device
	btfsc	c			;check to see if aknowledge
	goto	I2C_Busy		;bit set (0). if not error
	movfw	Temp_2
	movwf	I2C_Data		;load data back in I2C_Data
	call	I2C_Tx			;and transmit it
	btfsc	c			;check to see if aknowledge
	goto	I2C_Busy		;bit set (0). if not error
	call	I2C_Stop
	return
 
I2C_Busy
	; 	Used when device hasn't sent an aknowledge to slave address
 	call	I2C_Stop
	movfw	Temp_2			; Reload I2C data from Temp_1
	movwf	I2C_Data
	goto	Write_Byte		;loops until device accepts address
 
;**********************************************************************
;                                                                     
;	AT Keyboard Lookup Table for M50555-001SP
;
;**********************************************************************

M50555_Table
		movwf	PCL
		retlw	0x00	;	00		
		GOTO	_F9	;F9	 1
		retlw	0x00	;	 2
		GOTO	_F5	;F5 	 3
		GOTO	_F3	;F3	 4
		GOTO	_F1	;F1 	 5
		GOTO	_F2	;F2 	 6
		GOTO	_F12	;F12 	 7
		retlw	0x08	;	 8
		GOTO	_F10	;F10	 9
		GOTO	_F8	;F8	 A
		GOTO	_F6	;F6	 B
		GOTO	_F4	;F4	 C
		retlw	0x00	;Tab	 D
		retlw	0xed	;`	 E
		retlw	0x00	;	 F
		retlw	0x00	;	10		
		retlw	0x00	;L Alt	 1
		goto	_SHIFT	;L Sft 	 2
		retlw	0x00	; 	 3
		retlw	0x00	;L Cntl	 4
		retlw	0x34	;Q 	 5
		retlw	0x01	;1 	 6
		retlw	0x00	; 	 7
		retlw	0x00	;	 8
		retlw	0x00	;	 9
		retlw	0x3d	;Z	 A
		retlw	0x36	;S	 B
		retlw	0x24	;A	 C
		retlw	0x3a	;W	 D
		retlw	0x02	;2	 E
		retlw	0x00	;	 F
		retlw	0x00	;	20		
		retlw	0x26	;C	 1
		retlw	0x3b	;X	 2
		retlw	0x27	;D 	 3
		retlw	0x28	;E	 4
		retlw	0x04	;4 	 5
		retlw	0x03	;3 	 6
		retlw	0x00	; 	 7
		retlw	0x00	;	 8
		retlw	0xff	;Spc	 9
		retlw	0x39	;V	 A
		retlw	0x29	;F	 B
		retlw	0x37	;T	 C
		retlw	0x35	;R	 D
		retlw	0x05	;5	 E
		retlw	0x00	;	 F
		retlw	0x00	;	30		
		retlw	0x31	;N	 1
		retlw	0x25	;B	 2
		retlw	0x2b	;H 	 3
		retlw	0x2a	;G	 4
		retlw	0x3c	;Y 	 5
		retlw	0x06	;6 	 6
		retlw	0x00	; 	 7
		retlw	0x00	;	 8
		retlw	0x00	;	 9
		retlw	0x30	;M	 A
		retlw	0x2d	;J	 B
		retlw	0x38	;U	 C
		retlw	0x07	;7	 D
		retlw	0x08	;8	 E
		retlw	0x00	;	 F
		retlw	0x00	;	40		
		retlw	0xee	;,	 1
		retlw	0x2e	;K	 2
		retlw	0x2c	;I 	 3
		retlw	0x32	;O	 4
		retlw	0x40	;0 	 5
		retlw	0x09	;9 	 6
		retlw	0x00	; 	 7
		retlw	0x00	;	 8
		retlw	0xef	;.	 9
		retlw	0xac	;/	 A
		retlw	0x2f	;L	 B
		retlw	0xf7	;;	 C
		retlw	0x33	;P	 D
		retlw	0xab	;-	 E
		retlw	0x00	;	 F
		retlw	0x00	;	50		
		retlw	0x00	;	 1
		retlw	0xed	;'	 2
		retlw	0x00	; 	 3
		retlw	0x3e	;[	 4
		retlw	0xae	;= 	 5
		retlw	0x00	; 	 6
		retlw	0x00	; 	 7
		retlw	0x00	;Caps L	 8
		goto	_SHIFT	;Rght Sft9
		retlw	0x00	;Entr	 A
		retlw	0x3f	;]	 B
		retlw	0x00	;	 C
		retlw	0x00	;\	 D
		retlw	0x00	;	 E
		retlw	0x00	;	 F
		retlw	0x00	;	60		
		retlw	0x00	;	 1
		retlw	0x00	;	 2
		retlw	0x00	; 	 3
		retlw	0x00	;	 4
		retlw	0x00	; 	 5
		goto	_BCKSPC	;Bk Spc	 6
		retlw	0x00	; 	 7
		retlw	0x00	;	 8
		retlw	0x01	;KP 1	 9
		retlw	0x00	;	 A
		retlw	0x04	;KP 4	 B
		retlw	0x07	;KP 7	 C
		retlw	0x00	;	 D
		retlw	0x00	;	 E
		retlw	0x00	;	 F
		retlw	0x40	;KP 0	70		
		retlw	0xef	;KP .	 1
		retlw	0x02	;KP 2	 2
		retlw	0x05	;KP 5 	 3
		retlw	0x06	;KP 6	 4
		retlw	0x08	;KP 8 	 5
		goto	_ESCAPE	;Esc 	 6
		retlw	0x00	;Num 	 7
		goto	_F11	;F11	 8
		retlw	0xaa	;KP +	 9
		retlw	0x03	;KP 3	 A
		retlw	0xab	;KP -	 B
		retlw	0xad	;KP *	 C
		retlw	0x09	;KP 9	 D
		retlw	0x00	;Scrl	 E
		retlw	0x00	;	 F
		retlw	0x00	;	80		
		retlw	0x00	;	 1
		retlw	0x00	;	 2
		GOTO	_F7	;F7 	 3

Help_Reg_Table
	addwf	PCL,f		; Add offset to PCL
	retlw	b'00011101'	; H_Pos_LO		
	retlw	b'00000000'	; H_Pos_HI
	retlw	b'00011000'	; V_Pos_LO
	retlw	b'00000100'	; V_Pos_HI
	retlw	b'00010111'	; Video_Reg_LO
	retlw	b'00000000'	; Video_Reg_HI
	retlw	b'11011110'	; Cont_Reg_LO
Help_Reg_Table_End
	retlw	b'00001000'	; Cont_Reg_HI

	IF (high (Help_Reg_Table) != high (Help_Reg_Table_End))
		ERROR "Keyboard lookup table hits page boundary!"
	ENDIF
Help_Page
		movwf	PCL				; Jump to possition in table
		dt	"Caption Generator Help  "	; Line 1
		dt	"F1=Help    F7=Border    "	; Line 2
		dt	"F2=Colour  F8=B.Mode    "	; Line 3
		dt	"F3=Scrn Md F9=H.Size 1  "	; Line 4
		dt	"F4         F10=V.Size 1 "      ; Line 5
		dt	"F5=H.Pos   F11=H.Size 2+"	; Line 6
		dt	"F6=V.Pos   F12=V.Size 2+"	; Line 7
		dt	"Esc=Edit                "	; Line 8
		dt	"Home=Clear Screen       "	; Line 9
		dt	"  www.radio-kits.co.uk  "	; Line 10
		
		
	end




	
This test section beyond end so doesn't assemble!	
Help_Page
		addwf	pcl	; Jump to possition in table
		dt	0c,24,33,37,2c,32,31,ff,10,28,31,28,35,24,37,32,35,ff,11,28,2f,33,ff,ff	; Line 1
		dt	0f,01,ff,11,28,2f,33,ff,ff,ff,ff,0f,07,ff,0b,32,35,27,28,35,ff,ff,ff,ff	; Line 2
		dt	0f,02,ff,02,32,2f,32,38,35,ff,ff,0f,08,ff,0b,ef,16,32,27,28,ff,ff,ff,ff	; Line 3
		dt	0f,03,ff,1c,ef,16,32,27,28,ff,ff,0f,09,ff,11,ef,1c,2c,3d,28,ff,01,ff,ff	; Line 4
		dt	0f,04,ff,ff,ff,ff,ff,ff,ff,ff,ff,0f,01,00,ff,1f,ef,1c,2c,3d,28,ff,01,ff	; Line 5
		dt	0f,05,ff,11,ef,33,32,36,ff,ff,ff,0f,01,01,ff,11,ef,1c,2c,3d,28,ff,02,aa	; Line 6
		dt	0f,06,ff,1f,ef,33,32,36,ff,ff,ff,0f,01,02,ff,1f,ef,1c,2c,3d,28,ff,02,aa	; Line 7
		dt	0e,36,26,ff,0e,27,2c,37,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff	; Line 8
		dt	11,32,30,28,ff,0c,2f,28,24,35,ff,1c,26,35,28,28,31,ff,ff,ff,ff,ff,ff,ff	; Line 9
		dt	ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff,ff	; Line10


;###################################################################
;
; 		Display ROM contents
;
;###################################################################
Rom_Discover
	movlw	.10
	movwf	Temp_1		; Line counter

	movlw	.16		; Hex counter
	movwf	Temp_3

	movlw	0xa0
	movwf	Temp_4		; Character value
Test_Loop
	movfw	Temp_4
	movwf	KBD
	call	_OUTP
	incf	Temp_4,f	; Next character
	decfsz	Temp_3,f
	goto	Test_Loop
	movlw	.16
	movwf	Temp_3
	movlw	.8		; move pointer to next line
	addwf	Char_Poss,f
	decfsz	Temp_1,f
	goto	Test_Loop
	goto	$
;#####################################################################


